// Fill out your copyright notice in the Description page of Project Settings.


#include "OverloadModule.h"

#include "BehaviorTree/BTCompositeNode.h"
#include "Components/SphereComponent.h"
#include "GameFramework/Character.h"

void AOverloadModule::BeginPlay()
{
	Super::BeginPlay();
	
	this->SetActorTickInterval(0.2);
	PlayerChar = GetWorld()->GetFirstPlayerController()->GetCharacter();
	PlayerCam = Cast<UCameraComponent>(PlayerChar->GetComponentByClass(UCameraComponent::StaticClass()));
}


void AOverloadModule::Tick(float DeltaSeconds)
{
	Super::Tick(DeltaSeconds);

	CheckForPotentialHackTarget();
}

void AOverloadModule::CheckForPotentialHackTarget()
{
	FHitResult OutHit;
	FVector LT_Start = PlayerCam->GetComponentLocation();
	FVector LT_End = PlayerCam->GetComponentLocation() + (PlayerCam->GetForwardVector() * DeadBodyCheckDistance);

	DrawDebugLine(GetWorld(), LT_Start, LT_End, FColor::Red, false, 0.2, 0, 1);
	
	if (!GetWorld()->LineTraceSingleByChannel(OutHit, LT_Start, LT_End, ECC_Camera) || !(OutHit.GetActor()->ActorHasTag(FName("Enemy")) && OutHit.GetActor()->ActorHasTag(FName("Dead"))))
	{
		PotentialHackTarget = nullptr;
		DisplayNeedADeadEnemy();
		return;
	}

	if (HackedEnemies.Contains(OutHit.GetActor()->GetUniqueID()))
	{
		PotentialHackTarget = nullptr;
		DisplayAlreadyHacked();
	}
	
	DrawDebugLine(GetWorld(), LT_Start, OutHit.ImpactPoint, FColor::Green, false, 0.2, 0, 1);
	PotentialHackTarget = OutHit.GetActor();
	DisplayFoundPotentialTarget();
}


void AOverloadModule::Activate()
{
	if (GadgetCantBeUsed() || !IsValid(PotentialHackTarget)) return;
	
	Super::Activate();
	HackedEnemies.Add(PotentialHackTarget->GetUniqueID());
	ExplodeClosestAliveEnemy();
}

void AOverloadModule::FinishedUsing()
{
	Super::FinishedUsing();
}

void AOverloadModule::ExplodeClosestAliveEnemy()
{
	FActorSpawnParameters SpawnParams;
	SpawnParams.SpawnCollisionHandlingOverride = ESpawnActorCollisionHandlingMethod::AlwaysSpawn;
	AActor* SphereCollisionActor = GetWorld()->SpawnActor<AActor>(USphereComponent::StaticClass(), PlayerChar->GetActorLocation(), PlayerChar->GetActorRotation(), SpawnParams);
	USphereComponent* SphereCollision = Cast<USphereComponent>(SphereCollisionActor);

	SphereCollision->SetHiddenInGame(false);
	SphereCollision->SetSphereRadius(AreaOfEffectInMeters * 100, true);

	TArray<AActor*> OverlappingActors;
	TArray<AActor*> EnemiesInRadius;

	// Find all living enemies within the radius
	SphereCollision->GetOverlappingActors(OverlappingActors, ACharacter::StaticClass());
	for (AActor* OverlappingActor : OverlappingActors)
	{
		if (OverlappingActor->ActorHasTag(FName("Enemy")) && !OverlappingActor->ActorHasTag(FName("Dead"))) EnemiesInRadius.Add(OverlappingActor);
	}

	if (EnemiesInRadius.IsEmpty())
	{
		DisplayFailedToFindTarget();
		return;
	}

	// Find the closest Enemy in radius
	AActor* ClosestEnemy = EnemiesInRadius[0];
	float Dist = FVector::Distance(PlayerChar->GetActorLocation(), ClosestEnemy->GetActorLocation());
	UE_LOG(LogTemp, Warning, TEXT("Distacne To first Considered Enemy = %f"), Dist);
	if (EnemiesInRadius.Num() < 2)
	{
		SpawnExplosiveOnActor(ClosestEnemy, SpawnParams);
		return;
	}
	
	for (int i = 1; i < EnemiesInRadius.Num(); i++)
	{
		float ConsideredDist = FVector::Distance(PlayerChar->GetActorLocation(), EnemiesInRadius[i]->GetActorLocation());
		if (ConsideredDist <= Dist)
		{
			ClosestEnemy = EnemiesInRadius[i];
			Dist = ConsideredDist;
		}
	}

	SpawnExplosiveOnActor(ClosestEnemy, SpawnParams);
	
}

void AOverloadModule::SpawnExplosiveOnActor(AActor* ActortoExplode, FActorSpawnParameters SpawnParams)
{
	GetWorld()->SpawnActor<AActor>(Explosive, ActortoExplode->GetActorLocation(), ActortoExplode->GetActorRotation(), SpawnParams);
}

